@@ -45,6 +45,9 @@
 ;; Declare functions from org-social-relay
 (declare-function org-social-relay--fetch-feeds "org-social-relay" (callback))
 
+;; Declare functions from org-social-parser
+(declare-function org-social-parser--extract-mentioned-urls "org-social-parser" (text))
+
 ;; Declare variables to avoid compilation warnings
 (defvar org-social-max-post-age-days)
 (defvar org-social-max-concurrent-downloads)
@@ -130,7 +133,8 @@ If START-DATE is nil, returns CONTENT unchanged."
               ;; Parse and filter posts
               (goto-char header-end)
               (let ((filtered-posts '()))
-                (while (re-search-forward "^\\*\\*.*$" nil t)
+                ;; Match only level-2 posts (** followed by non-asterisk or newline)
+                (while (re-search-forward "^\\*\\*\\($\\|[^*]\\)" nil t)
                   (let ((post-start (line-beginning-position)))
                     ;; Find post ID
                     (when (re-search-forward ":ID:\\s-*\\(.+\\)$" nil t)
@@ -140,8 +144,9 @@ If START-DATE is nil, returns CONTENT unchanged."
                         (when (and (not (string< post-id start-date))
                                    (string< post-id (format-time-string "%FT%T%z")))
                           ;; Find post end (next ** or end of buffer)
+                          ;; Use ($|[^*]) to match ** at end of line or followed by non-asterisk
                           (let ((post-end (save-excursion
-                                            (if (re-search-forward "^\\*\\*.*$" nil t)
+                                            (if (re-search-forward "^\\*\\*\\($\\|[^*]\\)" nil t)
 						(line-beginning-position)
                                               (point-max)))))
                             (push (buffer-substring-no-properties post-start post-end)
@@ -343,6 +348,37 @@ When a download completes, the next pending item is automatically started."
       (message "All feeds downloaded!")
       (run-hooks 'org-social-after-fetch-posts-hook))))
 
+(defun org-social-feed--should-show-post (post)
+  "Determine if POST should be visible to the current user.
+Returns t if the post should be shown, nil otherwise.
+According to Org Social specification v1.5:
+- Posts without VISIBILITY property are public (always visible)
+- Posts with VISIBILITY:public are always visible
+- Posts with VISIBILITY:mention are only visible if:
+  * The user is the author, OR
+  * The user is mentioned in the post body
+- Posts with GROUP property ignore VISIBILITY (always visible to group members)"
+  (let ((visibility (alist-get 'visibility post))
+        (author-url (alist-get 'author-url post))
+        (text (alist-get 'text post))
+        (group (alist-get 'group post))
+        (my-url (alist-get 'url org-social-variables--my-profile)))
+    ;; Posts with GROUP property ignore VISIBILITY
+    (if group
+        t
+      ;; If no VISIBILITY property or VISIBILITY is "public", show the post
+      (if (or (not visibility)
+              (string= visibility "public"))
+          t
+        ;; If VISIBILITY is "mention", check if we're the author or mentioned
+        (when (string= visibility "mention")
+          (or
+           ;; We are the author
+           (and my-url author-url (string= my-url author-url))
+           ;; We are mentioned in the post body
+           (and my-url text
+                (member my-url (org-social-parser--extract-mentioned-urls text)))))))))
+
 (defun org-social-feed--get-timeline ()
   "Get all posts from all feeds sorted by date."
   (let* ((timeline (mapcan (lambda (feed)
@@ -367,6 +403,7 @@ When a download completes, the next pending item is automatically started."
                                           ;; Exclude group posts (posts with GROUP property)
                                           ;; Exclude reactions (posts with reply_to + mood)
                                           ;; Filter by language if org-social-language-filter is set
+                                          ;; Filter by visibility (VISIBILITY:mention)
                                           (let ((text (alist-get 'text post))
                                                 (group (alist-get 'group post))
                                                 (mood (alist-get 'mood post))
@@ -384,7 +421,9 @@ When a download completes, the next pending item is automatically started."
                                                  (or (null org-social-language-filter)
                                                      (and lang
                                                           (not (string-empty-p lang))
-                                                          (member lang org-social-language-filter))))))
+                                                          (member lang org-social-language-filter)))
+                                                 ;; Filter by visibility
+                                                 (org-social-feed--should-show-post post))))
                                         timeline))
          (timeline-sorted (sort timeline-filtered
                                 (lambda (a b)
