@@ -243,33 +243,20 @@ nil otherwise."
   (if org-social-mode
       (progn
         (org-mode)
-        (add-hook 'before-save-hook #'org-social-file--before-save nil t)
+        ;; Use depth 90 to run AFTER delete-trailing-whitespace (depth 0)
+        (add-hook 'before-save-hook #'org-social-file--before-save 90 t)
         (add-hook 'after-save-hook #'org-social-file--auto-save nil t))
     (remove-hook 'before-save-hook #'org-social-file--before-save t)
     (remove-hook 'after-save-hook #'org-social-file--auto-save t)))
 
-(defun org-social-file--normalize-empty-headers ()
-  "Add a space after empty headers (**, ***, ****, etc.) in the current buffer.
-This ensures that lines containing only asterisks become properly formatted.
-For example, '**' becomes '** ', '***' becomes '*** ', etc.
-Does NOT save the buffer - modifications happen in memory only."
-  (save-excursion
-    (goto-char (point-min))
-    ;; Match lines with only asterisks (2 or more) at the start
-    (while (re-search-forward "^\\(\\*\\{2,\\}\\)$" nil t)
-      ;; Found a line with only asterisks
-      (end-of-line)
-      (insert " "))))
+;; Removed org-social-file--normalize-empty-headers function
+;; No longer needed since all posts now have ID in header (Org Social v1.6)
 
 (defun org-social-file--before-save ()
-  "Normalize empty headers before saving."
-  (let ((current-file (buffer-file-name))
-        (target-file (if (org-social-file--is-vfile-p org-social-file)
-                         (org-social-file--get-local-file-path org-social-file)
-                       org-social-file)))
-    (when (and current-file
-               (file-equal-p current-file target-file))
-      (org-social-file--normalize-empty-headers))))
+  "Hook run before saving Org-social files."
+  ;; Currently no pre-save processing needed
+  ;; This function is kept for future use if needed
+  nil)
 
 (defun org-social-file--auto-save ()
   "Auto-save handler for Org-social files."
@@ -302,10 +289,11 @@ Does NOT save the buffer - modifications happen in memory only."
     (insert "\n* Posts")
     (point)))
 
-(defun org-social-file--insert-post-template (&optional reply-url reply-id group-context)
+(defun org-social-file--insert-post-template (&optional reply-url reply-id group-context visibility)
   "Insert a new post template at the current position.
 If REPLY-URL and REPLY-ID are provided, create a reply post.
-If GROUP-CONTEXT is provided, add GROUP property to the post."
+If GROUP-CONTEXT is provided, add GROUP property to the post.
+If VISIBILITY is \"mention\", add VISIBILITY property to the post."
   (let ((timestamp (org-social-parser--generate-timestamp))
         (lang-value (if (and (boundp 'org-social-default-lang)
                              org-social-default-lang
@@ -330,8 +318,7 @@ If GROUP-CONTEXT is provided, add GROUP property to the post."
               (insert "\n")
             ;; Not on new line, add two
             (insert "\n\n")))))
-    (insert "** \n:PROPERTIES:\n")
-    (insert (format ":ID: %s\n" timestamp))
+    (insert (format "** %s\n:PROPERTIES:\n" timestamp))
     ;; Only insert LANG if it has a value (optional field)
     (when lang-value
       (insert (format ":LANG: %s\n" lang-value)))
@@ -345,6 +332,9 @@ If GROUP-CONTEXT is provided, add GROUP property to the post."
             (relay-url (alist-get 'relay-url group-context)))
         (when (and group-name relay-url)
           (insert (format ":GROUP: %s %s\n" group-name relay-url)))))
+    ;; Add VISIBILITY property if visibility is "mention"
+    (when (and visibility (string= visibility "mention"))
+      (insert ":VISIBILITY: mention\n"))
     (insert ":MOOD: \n")
     (insert ":END:\n\n")
     (goto-char (point-max))))
@@ -377,8 +367,7 @@ and POLL-END is the RFC 3339 formatted end time."
               (insert "\n")
             ;; Not on new line, add two
             (insert "\n\n")))))
-    (insert "** \n:PROPERTIES:\n")
-    (insert (format ":ID: %s\n" timestamp))
+    (insert (format "** %s\n:PROPERTIES:\n" timestamp))
     ;; Only insert LANG if it has a value (optional field)
     (when lang-value
       (insert (format ":LANG: %s\n" lang-value)))
@@ -493,20 +482,28 @@ If REPLY-URL and REPLY-ID are provided, create a reply post.
 If GROUP-CONTEXT is provided, add GROUP property to the post."
   (let ((target-file (if (org-social-file--is-vfile-p org-social-file)
                          (org-social-file--get-local-file-path org-social-file)
-                       org-social-file)))
+                       org-social-file))
+        (visibility nil))
     (unless (and (buffer-file-name)
                  (string= (expand-file-name (buffer-file-name))
                           (expand-file-name target-file)))
-      (org-social-file--open)))
-  (save-excursion
-    (org-social-file--find-posts-section)
+      (org-social-file--open))
+    ;; Ask for visibility only if it's a new post (not a reply or group post)
+    (when (and (not reply-url) (not group-context))
+      (let ((choice (completing-read "Post visibility: "
+                                     '("public" "mention")
+                                     nil t nil nil "public")))
+        (when (string= choice "mention")
+          (setq visibility "mention"))))
+    (save-excursion
+      (org-social-file--find-posts-section)
+      (goto-char (point-max))
+      (org-social-file--insert-post-template reply-url reply-id group-context visibility))
     (goto-char (point-max))
-    (org-social-file--insert-post-template reply-url reply-id group-context))
-  (goto-char (point-max))
-  ;; Validate file after adding post
-  (when (fboundp 'org-social-validator-validate-and-display)
-    (require 'org-social-validator)
-    (org-social-validator-validate-and-display)))
+    ;; Validate file after adding post
+    (when (fboundp 'org-social-validator-validate-and-display)
+      (require 'org-social-validator)
+      (org-social-validator-validate-and-display))))
 
 (defun org-social-file--new-poll ()
   "Create a new poll in your Org-social feed.
@@ -609,8 +606,7 @@ EMOJI is the reaction emoji."
               (insert "\n")
             ;; Not on new line, add two
             (insert "\n\n")))))
-    (insert "** \n:PROPERTIES:\n")
-    (insert (format ":ID: %s\n" timestamp))
+    (insert (format "** %s\n:PROPERTIES:\n" timestamp))
     (insert ":CLIENT: org-social.el\n")
     (insert (format ":REPLY_TO: %s#%s\n" reply-url reply-id))
     ;; Add GROUP property if we're in a group context
@@ -672,8 +668,7 @@ Optional COMMENT is a text comment to add to the boost."
               (insert "\n")
             ;; Not on new line, add two
             (insert "\n\n")))))
-    (insert "** \n:PROPERTIES:\n")
-    (insert (format ":ID: %s\n" timestamp))
+    (insert (format "** %s\n:PROPERTIES:\n" timestamp))
     (insert ":CLIENT: org-social.el\n")
     (insert (format ":INCLUDE: %s#%s\n" post-url post-id))
     ;; Add GROUP property if we're in a group context
@@ -729,11 +724,69 @@ Returns nil if extraction fails."
 NICK is the user's nickname and URL is their social.org URL."
   (insert (format "[[org-social:%s][%s]]" url nick)))
 
+(defun org-social-file--get-mentions-cache-path ()
+  "Get the path to the mentions cache file."
+  (expand-file-name "org-social-mentions" user-emacs-directory))
+
+(defun org-social-file--save-mentions-cache (users)
+  "Save USERS list to mentions cache file.
+USERS is a list of cons cells (NICK . URL)."
+  (when users
+    (let ((cache-file (org-social-file--get-mentions-cache-path)))
+      (with-temp-file cache-file
+        (insert ";; Org-social mentions cache - Auto-generated file\n")
+        (insert ";; Do not edit manually.\n\n")
+        (insert "(")
+        (dolist (user users)
+          (insert (format "\n  (%S . %S)"
+                          (car user)  ; NICK
+                          (cdr user)))) ; URL
+        (insert "\n)\n")))))
+
+(defun org-social-file--load-mentions-cache ()
+  "Load mentions cache from file.
+Returns a list of cons cells (NICK . URL), or nil if cache doesn't exist."
+  (let ((cache-file (org-social-file--get-mentions-cache-path)))
+    (when (file-exists-p cache-file)
+      (condition-case err
+          (with-temp-buffer
+            (insert-file-contents cache-file)
+            (goto-char (point-min))
+            ;; Skip comment lines
+            (while (looking-at "^;;")
+              (forward-line 1))
+            ;; Read the s-expression
+            (read (current-buffer)))
+        (error
+         (message "Error loading mentions cache: %s" (error-message-string err))
+         nil)))))
+
+(defun org-social-file--update-mentions-cache-async ()
+  "Update mentions cache asynchronously from relay without blocking Emacs.
+This fetches ALL users from relay in background and saves them to cache."
+  (when (and (boundp 'org-social-relay)
+             org-social-relay
+             (not (string-empty-p org-social-relay)))
+    ;; Fetch ALL feed URLs from relay (just URLs, not full feeds)
+    (require 'org-social-relay)
+    (org-social-relay--fetch-feeds
+     (lambda (feeds-list)
+       (when feeds-list
+         ;; Now fetch user info for ALL feeds from relay
+         (org-social-user-queue-fetch-users
+          feeds-list
+          (lambda (users)
+            (when users
+              (let ((user-list (mapcar (lambda (user)
+                                         (cons (alist-get 'nick user)
+                                               (alist-get 'url user)))
+                                       users)))
+                (org-social-file--save-mentions-cache user-list))))))))))
+
 (defun org-social-file--get-relay-users (callback)
   "Get list of users from relay server and call CALLBACK with results.
 CALLBACK is called with a list of cons cells (NICK . URL)."
   (require 'org-social-relay)
-  (message "Loading users from relay...")
   (org-social-relay--fetch-feeds
    (lambda (feeds-list)
      (if feeds-list
@@ -747,6 +800,8 @@ CALLBACK is called with a list of cons cells (NICK . URL)."
                                            (cons (alist-get 'nick user)
                                                  (alist-get 'url user)))
                                          users)))
+                  ;; Save to cache for faster future access
+                  (org-social-file--save-mentions-cache user-list)
                   (funcall callback user-list))
               (message "No users could be fetched from relay")
               (funcall callback nil))))
@@ -754,47 +809,63 @@ CALLBACK is called with a list of cons cells (NICK . URL)."
        (funcall callback nil)))))
 
 (defun org-social-file--mention-user ()
-  "Prompt for a followed user and insert a mention at point."
+  "Prompt for a followed user and insert a mention at point.
+Uses cached user data for instant access when available."
   (interactive)
-  (if (and (boundp 'org-social-only-relay-followers-p)
-           org-social-only-relay-followers-p
-           (boundp 'org-social-relay)
-           org-social-relay
-           (not (string-empty-p org-social-relay)))
-      ;; Use relay to get users
-      (org-social-file--get-relay-users
-       (lambda (users)
-         (if users
-             ;; Run completing-read in the main thread context
-             (run-at-time 0 nil
-                          (lambda ()
-                            (let* ((user-alist (mapcar (lambda (user)
-                                                         (cons (car user) user))
-                                                       users))
-                                   (selected-nick (completing-read "Mention user: "
-                                                                   (mapcar #'car user-alist)
-                                                                   nil t))
-                                   (selected-user (cdr (assoc selected-nick user-alist))))
-                              (when selected-user
-                                (org-social-file--insert-mention (car selected-user)
-                                                                 (cdr selected-user))
-                                (message "Mentioned user: %s" (car selected-user))))))
-           (message "No users found in relay"))))
-    ;; Use local followers
-    (let ((followed-users (org-social-file--get-followed-users)))
-      (if followed-users
-          (let* ((user-alist (mapcar (lambda (user)
-                                       (cons (car user) user))
-                                     followed-users))
-                 (selected-nick (completing-read "Mention user: "
-                                                 (mapcar #'car user-alist)
-                                                 nil t))
-                 (selected-user (cdr (assoc selected-nick user-alist))))
-            (when selected-user
-              (org-social-file--insert-mention (car selected-user)
-                                               (cdr selected-user))
-              (message "Mentioned user: %s" (car selected-user))))
-        (message "No followed users found. Add users to your #+FOLLOW: list first.")))))
+  ;; Strategy: Try cache first (fast), fallback to fetching if needed
+  (let ((cached-users (org-social-file--load-mentions-cache)))
+    (if cached-users
+        ;; Cache exists - use it immediately (instant!)
+        (let* ((user-alist (mapcar (lambda (user)
+                                     (cons (car user) user))
+                                   cached-users))
+               (selected-nick (completing-read "Mention user: "
+                                               (mapcar #'car user-alist)
+                                               nil t))
+               (selected-user (cdr (assoc selected-nick user-alist))))
+          (when selected-user
+            (org-social-file--insert-mention (car selected-user)
+                                             (cdr selected-user))
+            (message "Mentioned user: %s" (car selected-user))))
+      ;; No cache - fall back to old behavior based on settings
+      (if (and (boundp 'org-social-relay)
+               org-social-relay
+               (not (string-empty-p org-social-relay)))
+          ;; Fetch from relay
+          (progn
+            (message "Fetching users from relay...")
+            (org-social-file--get-relay-users
+             (lambda (users)
+               (if users
+                   (run-at-time 0 nil
+                                (lambda ()
+                                  (let* ((user-alist (mapcar (lambda (user)
+                                                               (cons (car user) user))
+                                                             users))
+                                         (selected-nick (completing-read "Mention user: "
+                                                                         (mapcar #'car user-alist)
+                                                                         nil t))
+                                         (selected-user (cdr (assoc selected-nick user-alist))))
+                                    (when selected-user
+                                      (org-social-file--insert-mention (car selected-user)
+                                                                       (cdr selected-user))
+                                      (message "Mentioned user: %s" (car selected-user))))))
+                 (message "No users found in relay")))))
+        ;; Use local followers as last resort
+        (let ((followed-users (org-social-file--get-followed-users)))
+          (if followed-users
+              (let* ((user-alist (mapcar (lambda (user)
+                                           (cons (car user) user))
+                                         followed-users))
+                     (selected-nick (completing-read "Mention user: "
+                                                     (mapcar #'car user-alist)
+                                                     nil t))
+                     (selected-user (cdr (assoc selected-nick user-alist))))
+                (when selected-user
+                  (org-social-file--insert-mention (car selected-user)
+                                                   (cdr selected-user))
+                  (message "Mentioned user: %s" (car selected-user))))
+            (message "No followed users found. Add users to your #+FOLLOW: list first.")))))))
 
 ;; Forward declarations for wrapper functions
 (declare-function org-social-new-post "org-social" (&optional reply-url reply-id))
@@ -917,8 +988,7 @@ NEW-URL is the new account URL."
               (insert "\n")
             ;; Not on new line, add two
             (insert "\n\n")))))
-    (insert "** \n:PROPERTIES:\n")
-    (insert (format ":ID: %s\n" timestamp))
+    (insert (format "** %s\n:PROPERTIES:\n" timestamp))
     (insert ":CLIENT: org-social.el\n")
     (insert (format ":MIGRATION: %s %s\n" old-url new-url))
     (insert ":END:\n\n")
@@ -988,8 +1058,7 @@ Returns the number of replacements made."
           (goto-char match-start)
           ;; Check if we're in a code block
           (let ((in-code-block (org-social-file--in-code-block-p))
-                (line-start (line-beginning-position))
-                (line-end (line-end-position)))
+                (line-start (line-beginning-position)))
             ;; Check if this line contains :MIGRATION:
             (goto-char line-start)
             (let ((is-migration-line (looking-at "^:MIGRATION:")))
@@ -1034,8 +1103,7 @@ Returns an alist with keys 'old-url, 'new-url, and 'id, or nil if no migration f
 Updates our local social.org file if a migration is found."
   (when-let ((migration (org-social-file--find-migration-in-feed feed-content)))
     (let ((old-url (alist-get 'old-url migration))
-          (new-url (alist-get 'new-url migration))
-          (id (alist-get 'id migration)))
+          (new-url (alist-get 'new-url migration)))
       ;; Only process if the old URL matches the feed URL we're downloading
       (when (and old-url new-url (string= old-url feed-url))
         (let ((target-file (if (org-social-file--is-vfile-p org-social-file)
@@ -1059,6 +1127,131 @@ FEEDS-DATA is a list of (url . content) cons cells."
         (when (and url content)
           (org-social-file--process-remote-migration url content))))))
 
+;;; ID Migration from properties to header
+
+(defun org-social-file--migrate-ids-to-header ()
+  "Migrate post IDs from :PROPERTIES: drawer to header format.
+Finds all posts with ID in properties and moves them to the header
+according to Org Social v1.6 specification.  Posts with ID already
+in header are left unchanged.  Returns count of migrated posts."
+  (interactive)
+  (let ((migrated-count 0)
+        (skipped-count 0)
+        (error-count 0))
+    (save-excursion
+      (goto-char (point-min))
+      ;; Find the Posts section
+      (if (not (re-search-forward "^\\* Posts" nil t))
+          (message "No '* Posts' section found in buffer")
+        ;; Process each post
+        (while (re-search-forward "^\\*\\*\\($\\|[^*]\\)" nil t)
+          (let ((post-start (match-beginning 0))
+                id-from-header
+                id-from-properties
+                props-start
+                props-end
+                prop-section-start
+                post-end)
+            (save-excursion
+              ;; Check if ID is already in header
+              (beginning-of-line)
+              (when (looking-at "^\\*\\*\\s-+\\([0-9]\\{4\\}-[0-9]\\{2\\}-[0-9]\\{2\\}T[0-9]\\{2\\}:[0-9]\\{2\\}:[0-9]\\{2\\}[+-][0-9]\\{2\\}\\(:[0-9]\\{2\\}\\|[0-9]\\{2\\}\\)\\)")
+                (setq id-from-header (match-string 1)))
+
+              ;; Find the end of this post
+              (forward-line 1)
+              (setq props-start (point))
+              (if (re-search-forward "^\\*\\*\\($\\|[^*]\\)" nil t)
+                  (progn
+                    (beginning-of-line)
+                    (setq post-end (point)))
+                (setq post-end (point-max)))
+
+              ;; Look for ID in properties
+              (goto-char props-start)
+              (when (and (< (point) post-end)
+                         (re-search-forward ":PROPERTIES:" post-end t))
+                (forward-line 1)
+                (setq prop-section-start (point))
+                (when (re-search-forward ":END:" post-end t)
+                  (setq props-end (point))
+                  ;; Extract ID from properties
+                  (goto-char prop-section-start)
+                  (when (re-search-forward "^:ID:\\s-*\\(.+\\)$" props-end t)
+                    (setq id-from-properties (string-trim (match-string 1))))))
+
+              ;; Decide what to do
+              (cond
+               ;; Already has ID in header - skip
+               (id-from-header
+                (setq skipped-count (1+ skipped-count)))
+
+               ;; Has ID only in properties - migrate it
+               (id-from-properties
+                (condition-case err
+                    (progn
+                      ;; First delete the :ID: line from properties (before modifying buffer)
+                      (when (and prop-section-start props-end)
+                        (goto-char prop-section-start)
+                        (when (re-search-forward "^:ID:\\s-*\\(.+\\)$" props-end t)
+                          (delete-region (line-beginning-position)
+                                         (progn (forward-line 1) (point)))))
+
+                      ;; Now replace the header line
+                      (goto-char post-start)
+                      (beginning-of-line)
+                      (when (looking-at "^\\*\\*\\s-*$")
+                        (delete-region (point) (line-end-position))
+                        (insert (format "** %s" id-from-properties))
+
+                        (setq migrated-count (1+ migrated-count))))
+                  (error
+                   (setq error-count (1+ error-count))
+                   (message "Error migrating post at line %d: %s"
+                            (line-number-at-pos post-start)
+                            (error-message-string err)))))
+
+               ;; No ID at all - error
+               (t
+                (setq error-count (1+ error-count))
+                (message "Post at line %d has no ID"
+                         (line-number-at-pos post-start)))))))))
+
+    ;; Report results
+    (message "ID Migration complete: %d migrated, %d skipped (already in header), %d errors"
+             migrated-count skipped-count error-count)
+    migrated-count))
+
+(defun org-social-migrate-ids-to-header ()
+  "Migrate all post IDs from properties to header format interactively.
+This function updates your social.org file to use the new Org Social v1.6
+format where post IDs appear in the header instead of the properties drawer.
+
+Before migration:
+  **
+  :PROPERTIES:
+  :ID: 2025-01-05T10:00:00+0100
+  :END:
+
+After migration:
+  ** 2025-01-05T10:00:00+0100
+  :PROPERTIES:
+  :END:
+
+The function will:
+- Only migrate posts that have ID in properties but not in header
+- Skip posts that already have ID in header
+- Show statistics of how many posts were migrated
+- Preserve all other post content and properties
+
+It is recommended to save your file before running this command."
+  (interactive)
+  (when (or (not (buffer-modified-p))
+            (y-or-n-p "Buffer has unsaved changes.  Continue migration anyway? "))
+    (let ((count (org-social-file--migrate-ids-to-header)))
+      (when (> count 0)
+        (message "Successfully migrated %d post(s).  Remember to save the buffer." count)))))
+
 ;; Interactive functions with proper naming
 (defalias 'org-social-save-file #'org-social-file--save)
 (defalias 'org-social-mention-user #'org-social-file--mention-user)
